@@ -16,7 +16,6 @@ class _LoggerAdapter(logging.LoggerAdapter):
     def process(self, msg, kwargs):
         return time.strftime("[%H:%M:%S] == ") + msg, kwargs
 
-
 log = _LoggerAdapter(logging.getLogger("mkdocs.semos"), {})
 
 CONVERT_TOOL = os.getenv('CONVERT_TOOL', 'D:/Ceiba/pocs/markdown/bpmn-to-image-test/convert.bat')
@@ -43,24 +42,28 @@ class SemosPlugin(BasePlugin):
 
         img = re.compile(img_regex)
         src = re.compile(src_regex)
+        index = 0
 
         for item in img.finditer(html):
             src_match = src.search(item.group())
 
             if src_match:
                 log.debug(item.group())
+                index = index = 1
                 
                 # <img alt="Diagrama solucion!" src="imgs/aws.drawio.svg" title="Diagrama Solución" />
                 # <object data="imgs/componentes.drawio.svg" type="image/svg+xml" id="svg" width="100%" height="100%"></object>
                 image_file = src_match.group(1)
+                html_class = 'doc-svg'
 
                 if image_file.endswith('.bpmn'):
-                    image_file = image_file + '.svg'
+                    html_class = 'doc-svg-bpmn'
+                    if CONVERT_MODE == 'DEBUG':
+                        image_file = f"{BPMN_ENDPOINT}/?{page.url}{image_file}.svg"
+                    else:
+                        image_file = image_file + '.svg'
 
-                if CONVERT_MODE == 'DEBUG':
-                    html = html.replace(item.group(), f"<object data=\"{BPMN_ENDPOINT}/?{page.url}{image_file}\" type=\"image/svg+xml\" id=\"svg\" width=\"100%\" height=\"100%\"></object>")
-                else:
-                    html = html.replace(item.group(), f"<object data=\"{image_file}\" type=\"image/svg+xml\" id=\"svg\" width=\"100%\" height=\"100%\"></object>")
+                html = html.replace(item.group(), f"<object data=\"{image_file}\" type=\"image/svg+xml\" class=\"{html_class}\" id=\"svg{index}\" width=\"100%\" height=\"100%\"></object>")
 
         return html
 
@@ -81,10 +84,73 @@ class SemosPlugin(BasePlugin):
                 #if bpmn.suffix == '.bpmn':
                 log.debug(bpmn)
             
-                subprocess.call([CONVERT_TOOL, bpmn, f"{bpmn}.svg"])
+                self.convert_bpmn_to_svg(bpmn, f"{bpmn}.svg")
 
         return
 
+    def on_serve(self, server, config, builder):
+        if CONVERT_MODE == 'DEBUG':
+            log.info('onServe -- debug-mode')
+            data = {}
+            tmp_dir = tempfile.gettempdir().replace('\\', '/')
+            db_path = f"{tmp_dir}/bpmn-converted-files"
+            db_file = f"{db_path}/data.json"
+
+            if not os.path.exists(db_path):
+                os.mkdir(db_path)
+
+            log.info(f"open database -- {db_file}")
+            try:
+                with open(db_file) as infile:
+                    data = json.load(infile)
+            except Exception:
+                log.info(f"database not exist -- {db_file}")
+
+            old_app = server.get_app()
+
+            def new_app(environ, start_response):
+                path = environ['PATH_INFO'].encode('latin-1').decode('utf-8', 'ignore')
+
+                if path.startswith(BPMN_ENDPOINT):
+                    log.info(path)
+                    fixed_path = self.fix_path(environ['QUERY_STRING'])
+                    file = (SRC_PATH + fixed_path).replace('.svg', '')
+                    new_file = f"{db_path}/{file}.svg"
+
+                    try:
+                        new_path = Path(os.path.dirname(os.path.abspath(new_file)))
+                        new_path.mkdir(parents=True)
+                    except Exception:
+                        None
+
+                    self.update_database(file, new_file, data, db_file)
+
+                    cached_file = data[file]
+
+                    while not os.path.exists(cached_file['new_file']):
+                        log.debug(f"wait => {cached_file['new_file']}")
+                        time.sleep(0.100)
+                    log.debug(f"exist => {cached_file['new_file']}")
+
+                    rfile = open(cached_file['new_file'], "rb")
+
+                    start_response("200 OK", [])
+                    return wsgiref.util.FileWrapper(rfile)                    
+                    #return []
+                else:
+                    return old_app(environ, start_response)
+            
+            server.set_app(new_app)
+
+        return server
+
+###########################################################
+# OTHER FUNCTIONS
+###########################################################
+
+    def convert_bpmn_to_svg(self, file_in, file_out):
+        subprocess.call([CONVERT_TOOL, file_in, file_out])
+
     def fix_path(self, file_path):
         parts = file_path.split('/')
 
@@ -116,62 +182,15 @@ class SemosPlugin(BasePlugin):
             save = False
 
         if save:
-            subprocess.call([CONVERT_TOOL, file, new_file])
-
+            self.convert_bpmn_to_svg(file, new_file)
+            
             json_data = json.dumps(data)
             f = open(db_file, 'w')
             f.write(json_data)
             f.close()
             log.info('bpmn database updated!')
 
-    def on_serve(self, server, config, builder):
-        if CONVERT_MODE == 'DEBUG':
-            log.info('onServe -- debug-mode')
-            data = {}
-            tmp_dir = tempfile.gettempdir().replace('\\', '/')
-            db_path = f"{tmp_dir}/bpmn-converted-files"
-            db_file = f"{db_path}/data.json"
 
-            if not os.path.exists(db_path):
-                os.mkdir(db_path)
-
-            log.info(f"open database -- {db_file}")
-            try:
-                with open(db_file) as infile:
-                    data = json.load(infile)
-            except Exception:
-                log.info(f"database not exist -- {db_file}")
-
-            old_app = server.get_app()
-
-            def new_app(environ, start_response):
-                path = environ['PATH_INFO'].encode('latin-1').decode('utf-8', 'ignore')
-
-                if path.startswith(BPMN_ENDPOINT):
-                    fixed_path = self.fix_path(environ['QUERY_STRING'])
-                    file = (SRC_PATH + fixed_path).replace('.svg', '')
-                    new_file = f"{db_path}/{file}.svg"
-
-                    try:
-                        new_path = Path(os.path.dirname(os.path.abspath(new_file)))
-                        new_path.mkdir(parents=True)
-                    except Exception:
-                        None
-
-                    self.update_database(file, new_file, data, db_file)
-
-                    cached_file = data[file]
-                    rfile = open(cached_file['new_file'], "rb")
-
-                    start_response("200 OK", [])
-                    return wsgiref.util.FileWrapper(rfile)                    
-                    #return []
-                else:
-                    return old_app(environ, start_response)
-            
-            server.set_app(new_app)
-
-        return server
 #
 #    def on_pre_build(self, config):
 #        return
